/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include <dawn/dawn.h>
#include "../../assets/models/pokerchip.h"
#include "../../file/asset.h"
#include "../../display/shader.h"
#include "../../display/texture.h"

/**
 * Renders a stack of chips at a position.
 * 
 * @param count Count of chips to render (Determines the stack size).
 * @param x X position of the stack.
 * @param y Y position of the stack.
 * @param z Z position of the stack.
 * @param yaw Rotation (YAW) of the chips.
 */
void holdemRenderChipSet(int32_t count,float x,float y,float z,float yaw);

/**
 * Get the position for a stack of chips based on a seat.
 * 
 * @param seat Seat to get the position for.
 * @return Position for the seat that the chips should render to.
 */
pokerposition_t holdemRenderChipGetPosition(uint8_t seat);

/**
 * Init the poker chips renderer for each player.
 */
void holdemRenderChipInit();

/**
 * Render the poker chips.
 */
void holdemRenderChip();